//#define WIN31

//	Scan.cpp contains the lexical analyzer for the Windows PIC demo.
    
    
#include <string.h>
#include <ctype.h>

#include "defs.h"
#include "defs2.h" 

#include "tparser.h"

struct TABLE
	{
	char	*kw;
	int		ltp;
	};

const int NTOKENS = 14;

static TABLE kwtable[NTOKENS] = {
		{ "black",   BLACK		},
		{ "blue",	 BLUE		},
		{ "box",     BOX		},
		{ "define",  DEFINE		},
		{ "dotted",  DOTTED		},
		{ "draw",    DRAW		},
		{ "ellipse", ELLIPSE	},
		{ "gray",    GRAY		},
		{ "green",	 GREEN		},
		{ "line",    LINE		},
		{ "polygon", POLYGON	},
		{ "red",	 RED		},
		{ "solid",   SOLID		},
		{ "white",   WHITE		},
	};


int kwsearch( const char * s )
	{
	int type = IDENTIFIER;

	for ( int i = 0; i < NTOKENS; i++ )
		{
		int comp = strcmp( kwtable[i].kw, s );
		if ( comp < 0 )
			continue;
		else
			{
			if ( comp == 0 )
				type = kwtable[i].ltp;
			break;
			}
		}
	return type;
	}


//  The parser class member constructor TParser()
//  opens a user-specified file for input.

TParser::TParser(const char * filename )
	{
	input = new ifstream( filename );
	if ( *input == NULL )
			{
			MessageBox( NULL, "This PIC file could not be opened.", filename, MB_OK );
			exit( 1 );
			}
	symlst = NULL;
	clr_object( &anObject );
	input->get( nextchar );
	}


//  The parser class member function yystring()
//  converts a lexical token into a string value.

const char * TParser::yystring( int token )
	{
	switch( token )
		{
	case IDENTIFIER:
		return yylval.ch;
	case ';':
		return "semicolon";
	case ',':
		return "comma";
	case '.':
		return "period";
	case '(':
		return "left paren";
	case ')':
		return "right paren";
	default:
		if ( token < 256 )
			{
			char *op = new char[2];
			op[0] = token;
			op[1] = 0;
			return op;
			}
		else for ( int i = 0; i < NTOKENS; i++ )
			if ( kwtable[i].ltp == token )
				return kwtable[i].kw;		// keyword
		}

	return NULL;
	}


//  The parser class member function yymessage() communicates
//  syntax errors. First it shows a dialog box with the token
//  that was found, and then it shows another dialog box with
//  a list of the tokens that were expected at that position.

void TParser::yymessage( MESS err, int token )
	{
	char		mess[256];
	static int	first;

	switch( err )
		{
	case START:
		strcpy( mess, "Syntax error" );
		if ( token )
			{
			strcat( mess, " at token " );
			strcat( mess, yystring(token) );
			}
		strcat( mess, "." );
		first = 1;
		break;
	case EXPECT:
		if ( first )
			strcat( mess, " Expected:  " );
		else
			strcat( mess, ", " );
		strcat( mess, yystring(token) );
		first = 0;
		break;
	case SENDIT:
		strcat( mess, "." );
		MessageBox( NULL, mess, "Parser Message", MB_OK );
		break;
	case FATAL:
		MessageBox( NULL, "PCYACC table overflow", "Parser Message", MB_OK );
		exit( 1 );
		break;
	default:
		MessageBox( NULL, "Unknown error.", "Parser Message", MB_OK );
		break;
		}
	}


int TParser::yylex()
	{
	int		tokentype;

	while ( nextchar==' ' || nextchar=='\t' || nextchar=='\n' )
		input->get( nextchar );

	if ( nextchar == EOF )
		return( 0 );

	if (isdigit(nextchar) || nextchar=='+' || nextchar=='-')
		{
		int sign = 1;

		if ( nextchar == '+' )
			yylval.in = 0;
		else if ( nextchar == '-' )
			{
			sign = -1;
			yylval.in = 0;
			}
		else
			yylval.in = nextchar - '0';

		input->get( nextchar );
		while ( isdigit(nextchar) )
			{
			yylval.in = (10 * yylval.in) + (nextchar - '0');
			input->get( nextchar );
			}
		yylval.in = sign * yylval.in;
		tokentype = INTEGER;
		}
	else if ( isalpha(nextchar) )
		{
		char	temp[64];
		int		length = 0;

		do	{
			temp[length++] = nextchar;
			input->get( nextchar );
			}
			while ( isalnum(nextchar) );
		temp[length++] = '\0';
		yylval.ch = new char[length];
		strcpy( yylval.ch, temp );
		tokentype = kwsearch( temp );
		}
	else
		{
		tokentype = nextchar;
		input->get( nextchar );
		}

	return tokentype;
	}




  
