/**********************************************************************
*                                                                     *
*  ABXSYM.HPP: Header file for SYMBOL TABLE CLASS                     *
*                                                                     *
*  Version 1.0                                                        *
*                                                                     *
*  By Y. Jenny Luo                                                    *
*                                                                     *
*  PCYACC (R) is a software product of ABRAXAS SOFTWARE INC.          *
*  Copyright (C) 1986-1996 by ABRAXAS SOFTWARE INC.                   *
*                                                                     *
*  Reference: PCYACC Object Oriented Toolkit                          *
*             ABRAXAS SOFTWARE, INC                                   *
*                                                                     *
**********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* define type constant */
#define TYPE_VOID			 1
#define TYPE_CHAR			 2
#define TYPE_SHORT			 3
#define TYPE_INT			 4
#define TYPE_LONG			 5
#define TYPE_LONG_LONG		 6
#define TYPE_EXTRA_INT		 7
#define TYPE_UCHAR			 8	/* unsigned char */
#define TYPE_USHORT			 9  /* unsigned short */
#define TYPE_UINT			10  /* unsigned int */
#define TYPE_ULONG			11  /* unsigned long */
#define TYPE_EXTRA_UNIT		12  /* Nonstandard */
#define TYPE_FLOAT			13
#define TYPE_SHORT_DOUBLE	14
#define TYPE_DOUBLE			15
#define TYPE_LONG_DOUBLE	16
#define TYPE_EXTRA_FLOAT	17	/* Nonstandard float */
#define TYPE_ENUM			18
#define TYPE_UNION			19
#define TYPE_STRUCT			20
#define TYPE_CLASS			21	/* C++ only */
#define TYPE_DEFINED		22	/* Typedef name */
#define TYPE_EXTRA_PTR		23  /* Nonstandard */
#define TYPE_CONSTRUCTOR	24	/* C++ only */
#define TYPE_DESTRUCTOR		25	/* C++ only */

/* define attribute constant */
#define ATTR_CONST			 1
#define ATTR_VAR			 2
#define ATTR_FUNCT			 3

#define ABX_TEXT        1024
union ABXVALUE
{
	int i;
	double db;
	char ch[ABX_TEXT];
	/* user can add more members according to their needs */
};
struct ABXsymtabentry
{
	char name[ABX_TEXT];
	int attribute;
	int type;
	ABXVALUE value;
	int scope;
	struct ABXsymtabentry *next;
};


class ABXSymbolTable
{
public:
	ABXSymbolTable();
	~ABXSymbolTable();
	void initSymbolTable(void);
    int setScope(int scp);
	int getScope(void);
	int maxScope(void);
	int createScope(int scp);
	int cleanScope(int scp);
	int addSymbol(char *name, int attribute, 
		int type, ABXVALUE value, int scp);
	struct ABXsymtabentry *lookupSymbol(char *s, int scp);
	int deleteSymbol(char *name, int scp);
	int setSymbolAttribute(char *name, int scp, int attribute);
	int setSymbolType(char *name, int scp, int type);
	int setSymbolValue(char *name, int scp, ABXVALUE value);
	int getSymbolAttribute(char *name, int scp);
	int getSymbolType(char *name, int scp, int type);
	union ABXVALUE getSymbolValue(char *name, int scp, ABXVALUE value);
	int insertSymbol(ABXsymtabentry *symbol);

private:
	int scope_indicator;
	struct ABXsymtab
	{
		int scope;  /* as keyword for the whole symbol table link list */
		struct ABXsymtabentry *symbol_list;
		struct ABXsymtab *next;
	};
	struct ABXsymtab *symtab_head, *symtab_ptr;
};
