
/*
 * SACALC EXAMPLE
 */

%{

/*#define YYSTYPE double    /* data type of yacc stack */

#define STAND_ALONE	1    
#define QUIT    ((double)101010)
#define LF 0x0a /* line feed character */
#define CR 0xd  /* carriage return */

%}
%union{
double db;
}

%token NEWLINE
%token NUMBER
%left '+' '-'   /* left associative, same precedence */
%left '*' '/'   /* left associative, higher precedence */
%left UNARYMINUS  

%%

list:    /* nothing */
	 { prompt(); }
       | list NEWLINE /* change ALL '\n' to token newline */
	 { prompt(); }
       | list expr NEWLINE
	 { if ( $2.db == QUIT) {
	      return(0);
	   } else {
	     System.out.println("  RESULT ====>" + $2.db);
	     System.out.flush();
	     prompt();
	   }
	 }
       | list error NEWLINE
	   { yyerrok;
	     prompt();
	   }
       ;
expr:    NUMBER              { $$.db = $1.db; }
       | '-' expr %prec UNARYMINUS { $$.db = -$2.db; }
       | expr '+' expr       { $$.db = $1.db + $3.db; }
       | expr '-' expr       { $$.db = $1.db - $3.db; }
       | expr '*' expr       { $$.db = $1.db * $3.db; }
       | expr '/' expr       { $$.db = $1.db / $3.db; }
       | '(' expr ')'        { $$.db = $2.db; }
       ;
%%

public static void main(String args[])
{
  if (args.length>0)
    System.out.println("nonmeaningful arguments");

  System.out.println("*************************************************");
  System.out.println("*  SACALC: A Simple Arithmetic Calculator       *");
  System.out.println("*                                               *");
  System.out.println("*  1) at the prompt READY>                      *");
  System.out.println("*     you type in an expression, e.g. 1+2<CR>   *");
  System.out.println("*     SACALC will evaluate the expression       *");
  System.out.println("*     and the result is displayed               *");
  System.out.println("*  2) to terminate the program                  *");
  System.out.println("*     type QUIT                                 *");
  System.out.println("*  3) if you make a mistake                     *");
  System.out.println("*     SACALC will complain and start over again *");
  System.out.println("*                                               *");
  System.out.println("*************************************************");
  
  yyparse();

  System.out.println("*************************************************");
  System.out.println("*  SACALC: A Simple Arithmetic Calculator       *");
  System.out.println("*                                               *");
  System.out.println("*  normal termination -- bye!                   *");
  System.out.println("*************************************************");
}

private static PushbackInputStream in = 
  new PushbackInputStream(System.in);

private static int yylex()
{
  int c = 0;
  double intpart = 0;
  double decpart = 0;
  int deccnt = 0;

  try
  {
    c = in.read();      // the try and catch pair can be put on whole 
  }                     // function body to reduce the number of the pairs.
  catch (IOException e)
  {
    System.out.println(e);
	System.exit(-1);
  }

  while (c == ' ' || c == '\t' )
  {
    try
	{
	  c = in.read();
	}
	catch (IOException e)
	{
	  System.out.println(e);
	  System.exit(-1);
	}
  } 

  if (c == -1)
    return 0;

  if (c == '.' || c <= '9' && c >= '0')
  {
    if (c == '.')
	{
	  try
	  {
	    c = in.read();
	  }
	  catch (IOException e)
	  {
	    System.out.println(e);
		System.exit(-1);
	  }

	  if (c > '9' || c < '0')
	  {
	    try
		{
		  in.unread(c);
		}
		catch (IOException e)
		{
		  System.out.println(e);
		  System.exit(-1);
		}
		return '?';
	  }

      while (c <='9' && c >= '0')
	  {
	    decpart = decpart * 10 + c -'0';
		deccnt++;
		try
		{
		  c = in.read();
		}
		catch (IOException e)
		{
		  System.out.println(e);
		  System.exit(-1);
		}
	  }

	  try
	  {
	    in.unread(c);
	  }
	  catch (IOException e)
	  {
	    System.out.println(e);
		System.exit(-1);
	  }

	  for (int k=0; k<deccnt; k++)
	  {
	    decpart = decpart/10;
	  }

	  yylval.db = decpart;
	}
	else
	{
	  while (c<='9' && c>='0')
	  {
	    intpart = intpart * 10 + c - '0';
		try
		{
		  c = in.read();
		}
		catch (IOException e)
		{
		  System.out.println(e);
	  System.exit(-1);
		}
	  }

	  if (c == '.')
	  {
	    deccnt = 0;
		try
		{
		  c = in.read();
		}
		catch (IOException e)
		{
		  System.out.println(e);
		  System.exit(-1);
		}

		if (c>'9' || c<'0')
		{
		  yylval.db = intpart;
		  try
		  {
		    in.unread(c);
		  }
		  catch(IOException e)
		  {
		    System.out.println(e);
			System.exit(-1);
		  }
		}
		else
		{
		  while (c>='0' && c<='9')
		  {
		    decpart = decpart * 10 + c - '0';
			deccnt++;
			try
			{
			  c = in.read();
			}
			catch (IOException e)
			{
			  System.out.println(e);
			  System.exit(-1);
			}
		  }

		  try
		  {
		    in.unread(c);
		  }
		  catch (IOException e)
		  {
		    System.out.println(e);
			System.exit(-1);
		  }

		  for (int k=0; k<deccnt; k++)
		  {
		    decpart = decpart/10;
		  }

		  yylval.db = intpart + decpart;
		}
	  }
	  else
	  {
	    try
		{
		  in.unread(c);
		}
		catch (IOException e)
		{
		  System.out.println(e);
		  System.exit(-1);
		}

		yylval.db = intpart;
	  }
	}
	return NUMBER;
  }

  if ( c==LF || c==CR )
  {
    if ( c == CR ) c = LF;
    pcyylineno++;
    return( NEWLINE );
  }
   
  if (c=='Q' || c == 'q')
  {
    try
	{
	  c = in.read();
	}
	catch (IOException e)
	{
	  System.out.println(e);
	  System.exit(-1);
	}

	if (c=='U' || c == 'u')
	{
	  try
	  {
	    c = in.read();
	  }
	  catch (IOException e)
	  {
	    System.out.println(e);
		System.exit(-1);
	  }

	  if (c=='I' || c=='i')
	  {
	    try
		{
		  c = in.read();
		}
		catch (IOException e)
		{
		  System.out.println(e);
		  System.exit(-1);
		}

		if (c=='T' || c== 't')
		{
		  yylval.db = QUIT;
		  return -1;
		}
		else
		{
		  return '?';
		}
		
	  }
	  else
	  {
	    return '?';
	  }
	}
	else
	{
	  return '?';
	}
  }

  return c;
}


private static void prompt()
{
  System.out.print("READY> ");
  System.out.flush();
}

private static void yyerror(String s, String t)
{
	System.out.println(s+" near line " + pcyylineno);
}

private static String yydisplay(int ch)
{
	int i=0, tok_index = 0;
	int tok_num = 2;
	String[] tok = new String[tok_num]; 

	tok[tok_index++] = "NEWLINE";
	tok[tok_index++] = "NUMBER";

    switch (ch)
    {
        case             0:   return ("[end of file]");
		case          '\b':   return (    "'\\b'"    );
		case          '\f':   return (    "'\\f'"    );
		case          '\n':   return (    "'\\n'"    );
		case          '\r':   return (    "'\\r'"    );
		case          '\t':   return (    "'\\t'"    );
		case     YYERRCODE:   return (   "[error]"   );
    }

    if(ch > YYERRCODE && ch <= YYERRCODE + tok_num)
    {
        return (tok[ch - (YYERRCODE + 1)]);                  /* is %token */
    }
    return "char \""+(char)ch+"\"";
}
